import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import { CandidateProps, CreateCandidateProps } from './candidate.types';
import { CarpoolStepProps } from './value-objects/carpool-step.value-object';
import { StepProps } from './value-objects/step.value-object';
import { ScheduleItem } from './value-objects/schedule-item.value-object';
import { Journey } from './value-objects/journey.value-object';
import { CalendarTools } from './calendar-tools.service';
import { JourneyItem } from './value-objects/journey-item.value-object';

export class CandidateEntity extends AggregateRoot<CandidateProps> {
  protected readonly _id: AggregateID;

  static create = (create: CreateCandidateProps): CandidateEntity => {
    const props: CandidateProps = { ...create };
    return new CandidateEntity({ id: create.id, props });
  };

  setCarpoolPath = (carpoolSteps: CarpoolStepProps[]): CandidateEntity => {
    this.props.carpoolSteps = carpoolSteps;
    return this;
  };

  setMetrics = (distance: number, duration: number): CandidateEntity => {
    this.props.distance = distance;
    this.props.duration = duration;
    return this;
  };

  setSteps = (steps: StepProps[]): CandidateEntity => {
    this.props.steps = steps;
    return this;
  };

  isDetourValid = (): boolean =>
    this._validateDistanceDetour() && this._validateDurationDetour();

  createJourneys = (fromDate: string, toDate: string): CandidateEntity => {
    this.props.driverJourneys = this.props.driverSchedule
      .map((driverScheduleItem: ScheduleItem) =>
        this._createJourney(fromDate, toDate, driverScheduleItem),
      )
      .filter(
        (journey: Journey | undefined) => journey !== undefined,
      ) as Journey[];
    this.props.passengerJourneys = this.props.passengerSchedule
      .map((passengerScheduleItem: ScheduleItem) =>
        this._createJourney(fromDate, toDate, passengerScheduleItem),
      )
      .filter(
        (journey: Journey | undefined) => journey !== undefined,
      ) as Journey[];
    return this;
  };

  private _validateDurationDetour = (): boolean =>
    this.props.duration
      ? this.props.duration <=
        this.props.driverDuration *
          (1 + this.props.spacetimeDetourRatio.maxDurationDetourRatio)
      : false;

  private _validateDistanceDetour = (): boolean =>
    this.props.distance
      ? this.props.distance <=
        this.props.driverDistance *
          (1 + this.props.spacetimeDetourRatio.maxDistanceDetourRatio)
      : false;

  private _createJourney = (
    fromDate: string,
    toDate: string,
    scheduleItem: ScheduleItem,
  ): Journey | undefined =>
    new Journey({
      day: scheduleItem.day,
      firstDate: CalendarTools.firstDate(scheduleItem.day, fromDate, toDate),
      lastDate: CalendarTools.lastDate(scheduleItem.day, fromDate, toDate),
      journeyItems: this._createJourneyItems(scheduleItem),
    });

  private _createJourneyItems = (
    scheduleItem: ScheduleItem,
  ): JourneyItem[] => [];

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
