import { ArgumentInvalidException } from '@mobicoop/ddd-library';
import { Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { ActorTime } from '@modules/ad/core/domain/value-objects/actor-time.value-object';

describe('Actor time value object', () => {
  it('should create an actor time value object', () => {
    const actorTimeVO = new ActorTime({
      role: Role.DRIVER,
      target: Target.START,
      firstDatetime: new Date('2023-09-01 07:00'),
      firstMinDatetime: new Date('2023-09-01 06:45'),
      firstMaxDatetime: new Date('2023-09-01 07:15'),
      lastDatetime: new Date('2024-08-30 07:00'),
      lastMinDatetime: new Date('2024-08-30 06:45'),
      lastMaxDatetime: new Date('2024-08-30 07:15'),
    });
    expect(actorTimeVO.role).toBe(Role.DRIVER);
    expect(actorTimeVO.target).toBe(Target.START);
    expect(actorTimeVO.firstDatetime.getHours()).toBe(7);
    expect(actorTimeVO.firstMinDatetime.getMinutes()).toBe(45);
    expect(actorTimeVO.firstMaxDatetime.getMinutes()).toBe(15);
    expect(actorTimeVO.lastDatetime.getHours()).toBe(7);
    expect(actorTimeVO.lastMinDatetime.getMinutes()).toBe(45);
    expect(actorTimeVO.lastMaxDatetime.getMinutes()).toBe(15);
  });
  it('should throw an error if dates are inconsistent', () => {
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01 07:00'),
          firstMinDatetime: new Date('2023-09-01 07:05'),
          firstMaxDatetime: new Date('2023-09-01 07:15'),
          lastDatetime: new Date('2024-08-30 07:00'),
          lastMinDatetime: new Date('2024-08-30 06:45'),
          lastMaxDatetime: new Date('2024-08-30 07:15'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01 07:00'),
          firstMinDatetime: new Date('2023-09-01 06:45'),
          firstMaxDatetime: new Date('2023-09-01 06:55'),
          lastDatetime: new Date('2024-08-30 07:00'),
          lastMinDatetime: new Date('2024-08-30 06:45'),
          lastMaxDatetime: new Date('2024-08-30 07:15'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01 07:00'),
          firstMinDatetime: new Date('2023-09-01 06:45'),
          firstMaxDatetime: new Date('2023-09-01 07:15'),
          lastDatetime: new Date('2024-08-30 07:00'),
          lastMinDatetime: new Date('2024-08-30 07:05'),
          lastMaxDatetime: new Date('2024-08-30 07:15'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01 07:00'),
          firstMinDatetime: new Date('2023-09-01 06:45'),
          firstMaxDatetime: new Date('2023-09-01 07:15'),
          lastDatetime: new Date('2024-08-30 07:00'),
          lastMinDatetime: new Date('2024-08-30 06:45'),
          lastMaxDatetime: new Date('2024-08-30 06:35'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2024-08-30 07:00'),
          firstMinDatetime: new Date('2024-08-30 06:45'),
          firstMaxDatetime: new Date('2024-08-30 07:15'),
          lastDatetime: new Date('2023-09-01 07:00'),
          lastMinDatetime: new Date('2023-09-01 06:45'),
          lastMaxDatetime: new Date('2023-09-01 07:15'),
        }),
    ).toThrow(ArgumentInvalidException);
    expect(
      () =>
        new ActorTime({
          role: Role.DRIVER,
          target: Target.START,
          firstDatetime: new Date('2023-09-01 07:00'),
          firstMinDatetime: new Date('2023-09-01 06:45'),
          firstMaxDatetime: new Date('2023-09-01 07:15'),
          lastDatetime: new Date('2024-08-31 07:00'),
          lastMinDatetime: new Date('2024-08-31 06:45'),
          lastMaxDatetime: new Date('2024-08-31 06:35'),
        }),
    ).toThrow(ArgumentInvalidException);
  });
});
