import {
  CalendarTools,
  CalendarToolsException,
} from '@modules/ad/core/domain/calendar-tools.service';

describe('Calendar tools service', () => {
  describe('First date', () => {
    it('should return the first date for a given week day within a date range', () => {
      const firstDate: Date = CalendarTools.firstDate(
        1,
        '2023-08-31',
        '2023-09-07',
      );
      expect(firstDate.getDay()).toBe(1);
      expect(firstDate.getDate()).toBe(4);
      expect(firstDate.getMonth()).toBe(8);
      const secondDate: Date = CalendarTools.firstDate(
        5,
        '2023-08-31',
        '2023-09-07',
      );
      expect(secondDate.getDay()).toBe(5);
      expect(secondDate.getDate()).toBe(1);
      expect(secondDate.getMonth()).toBe(8);
      const thirdDate: Date = CalendarTools.firstDate(
        4,
        '2023-08-31',
        '2023-09-07',
      );
      expect(thirdDate.getDay()).toBe(4);
      expect(thirdDate.getDate()).toBe(31);
      expect(thirdDate.getMonth()).toBe(7);
    });
    it('should throw an exception if a given week day is not within a date range', () => {
      expect(() => {
        CalendarTools.firstDate(1, '2023-09-05', '2023-09-07');
      }).toThrow(CalendarToolsException);
    });
    it('should throw an exception if a given week day is invalid', () => {
      expect(() => {
        CalendarTools.firstDate(8, '2023-09-05', '2023-09-07');
      }).toThrow(CalendarToolsException);
    });
  });

  describe('Second date', () => {
    it('should return the last date for a given week day within a date range', () => {
      const firstDate: Date = CalendarTools.lastDate(
        0,
        '2023-09-30',
        '2024-09-30',
      );
      expect(firstDate.getDay()).toBe(0);
      expect(firstDate.getDate()).toBe(29);
      expect(firstDate.getMonth()).toBe(8);
      const secondDate: Date = CalendarTools.lastDate(
        5,
        '2023-09-30',
        '2024-09-30',
      );
      expect(secondDate.getDay()).toBe(5);
      expect(secondDate.getDate()).toBe(27);
      expect(secondDate.getMonth()).toBe(8);
      const thirdDate: Date = CalendarTools.lastDate(
        1,
        '2023-09-30',
        '2024-09-30',
      );
      expect(thirdDate.getDay()).toBe(1);
      expect(thirdDate.getDate()).toBe(30);
      expect(thirdDate.getMonth()).toBe(8);
    });
    it('should throw an exception if a given week day is not within a date range', () => {
      expect(() => {
        CalendarTools.lastDate(2, '2024-09-27', '2024-09-30');
      }).toThrow(CalendarToolsException);
    });
    it('should throw an exception if a given week day is invalid', () => {
      expect(() => {
        CalendarTools.lastDate(8, '2023-09-30', '2024-09-30');
      }).toThrow(CalendarToolsException);
    });
  });
});
